import warnings

from .octicons import OcticonStore

try:
    default_store = OcticonStore.from_file(None)
except Exception as e:
    default_store = None
    warnings.warn("Failed to load Octicons store.", source=e)



import functools
import itertools
import json
import pathlib

from ._escape_utils import conditional_escape, mark_safe


class Octicon():

    def __init__(self, data):
        self.width = data['width']
        self.height = data['height']
        self.name = data['name']
        self.keywords = data['keywords']
        self.path = data['path']

    @staticmethod
    def key_exists(opts, key):
        return opts.get(key, None) is not None

    @staticmethod
    def escape_attr(key, value):
        if value is True:
            return key
        elif value is False or value is None:
            return ''
        elif key == 'class':
            if not isinstance(value, str):
                value = ' '.join(map(str.strip, value))
        return '%s="%s"' % (key, conditional_escape(value))

    def as_html(self, **opts):
        raw_attrs = self.build_attributes(opts)
        attrs = ' '.join(filter(None, itertools.starmap(
            self.escape_attr, raw_attrs.items())))
        return mark_safe("<svg %s>%s</svg>" % (attrs, self.path))

    def build_attributes(self, opts):
        key_exists = functools.partial(self.key_exists, opts)
        defaults = self.default_attrs
        opts = {k.replace('_', '-'): v for k, v in opts.items()}
        overrides = {}
        if key_exists('width') and not key_exists('height'):
            # Generate the correct height from the aspect ratio
            overrides['height'] = opts["width"] / self.size_ratio
        elif key_exists('height') and not key_exists('width'):
            # Generate the correct width from the aspect ratio
            overrides['width'] = opts["height"] * self.size_ratio
        if key_exists('class'):
            if isinstance(opts['class'], str):
                # A string.
                overrides["class"] = [
                    *defaults['class'],
                    *opts['class'].split(),
                ]
            else:
                # An iterable of some sort.
                overrides["class"] = [
                    *defaults['class'],
                    *opts['class'],
                ]
        if key_exists('aria-label'):
            overrides["aria-label"] = opts["aria-label"]
            overrides["role"] = "img"
            # Un-hide the icon
            del defaults["aria-hidden"]
        out_attrs = dict(defaults)
        out_attrs.update(opts)
        out_attrs.update(overrides)
        return {
            key: value
            for key, value in out_attrs.items()
            # If we've defined custom rules on what is considered 'present'
            # Apply them here as well so that they're applied to all values
            if self.key_exists(out_attrs, key)
        }

    @property
    def size_ratio(self):
        return self.width / self.height

    @property
    def default_attrs(self):
        return {
            "version": "1.1",
            "width": self.width,
            "height": self.height,
            "viewBox": "0 0 %s %s" % (self.width, self.height),
            "class": ["octicon", "octicon-" + self.name],
            "aria-hidden": "true"
        }


class OcticonStore():

    @classmethod
    def from_file(cls, file=None):
        if file is None:
            file = pathlib.Path(__file__).parent / 'data.json'
        else:
            file = pathlib.Path(file)
        return cls(json.load(file.open()))

    def __init__(self, data_dict, *, Octicon=Octicon):
        raw_icons = [
            Octicon(data)
            for data in data_dict.values()
        ]
        self.icons = dict([
            (icon.name, icon)
            for icon in raw_icons
        ] + [
            (keyword, icon)
            for icon in raw_icons
            for keyword in icon.keywords
        ])

    def get_icon(self, name):
        return self.icons[name]

    def __getitem__(self, name):
        return self.get_icon(name)

    def get_as_html(self, name, **opts):
        return self[name].as_html(**opts)





class SafeHtmlString(str):

    def __html__(self):
        return self


_mark_safe = SafeHtmlString


def _conditional_escape(text):
    # Taken from django.utils.html.conditional_escape (Django version 2.0.8)
    if hasattr(text, '__html__'):
        return text.__html__()
    else:
        return _escape(text)


def _escape(text):
    # Taken from django.utils.html.escape (Django version 2.0.8)
    return _mark_safe(
        _force_text(text).replace('&', '&amp;').replace('<', '&lt;')
        .replace('>', '&gt;').replace('"', '&quot;').replace("'", '&#39;')
    )


def _force_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    # Adapted from django.utils.encoding.force_text (Django version 2.0.8)
    if issubclass(type(s), str):
        return s
    if isinstance(s, bytes):
        return str(s, encoding, errors)
    else:
        return str(s)


try:
    # Support no django; but prefer django if it's avaliable
    from django.utils.encoding import force_text
    from django.utils.html import conditional_escape, escape
    from django.utils.safestring import mark_safe
except:
    conditional_escape = _conditional_escape
    escape = _escape
    mark_safe = _mark_safe
    force_text = _force_text



from django import template
from octicons.store import default_store

assert default_store is not None

register = template.Library()


@register.simple_tag
def octicon(name, **options):
    return default_store.get_as_html(name, **options)



from setuptools import find_packages, setup

packages = find_packages(
    where='./',
    include=['octicons', 'octicons.*'],
)
if not packages:
    raise ValueError('No packages detected.')


with open('./README.md', 'r') as readme_file:
    readme = readme_file.read()

setup(
    name='octicons',
    version='0.1',
    description='A Python port of GitHub\'s Octicons, with Django support',
    long_description=readme,
    long_description_content_type='text/markdown',
    python_requires='>=3.4',
    url='https://github.com/leesdolphin/octicons',
    author='Opal Symes',
    author_email='python@opal.codes',
    license='LGPLv3+',
    packages=packages,
    include_package_data=True,
    install_requires=[],
    zip_safe=False,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
